"""Views for the ``test_app`` app."""
from django.contrib.contenttypes.models import ContentType
from django.views.generic import TemplateView

from .models import DummyGallery


class HomeView(TemplateView):
    template_name = "home.html"

    def get_context_data(self):
        context = super(HomeView, self).get_context_data()
        try:
            context.update({
                'dummy_gallery': DummyGallery.objects.all()[0],
                'gallery_ctype': ContentType.objects.get_for_model(
                    DummyGallery.objects.all()[0]).model,
            })
        except IndexError:
            pass
        return context



"""Views for the ``django-user-media`` app."""
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.db.models import ObjectDoesNotExist
from django.http import Http404, HttpResponse
from django.template.loader import render_to_string
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.generic import CreateView, DeleteView, FormView, UpdateView

from django_libs.views_mixins import AjaxResponseMixin
from easy_thumbnails.files import get_thumbnailer
from simplejson import dumps

from user_media.forms import UserMediaImageForm, UserMediaImageSingleUploadForm
from user_media.models import UserMediaImage


class UserMediaImageViewMixin(object):
    """
    Mixin for views that deal with `UserMediaImage` objects.

    When using this mixin please make sure that you call `_add_next_and_user()`
    in your `dispatch()` method.

    """
    model = UserMediaImage

    def _add_next_and_user(self, request):
        self.next = request.POST.get('next', '') or request.GET.get('next', '')
        self.user = request.user

    def get_context_data(self, **kwargs):
        """
        Adds `next` to the context.

        This makes sure that the `next` parameter doesn't get lost if the
        form was submitted invalid.

        """
        ctx = super(UserMediaImageViewMixin, self).get_context_data(**kwargs)
        ctx.update({
            'action': self.action,
            'next': self.next,
        })
        return ctx

    def get_success_url(self):
        """
        Returns the success URL.

        This is either the given `next` URL parameter or the content object's
        `get_absolute_url` method's return value.

        """
        if self.next:
            return self.next
        if self.object and self.object.content_object:
            return self.object.content_object.get_absolute_url()
        raise Exception(
            'No content object given. Please provide ``next`` in your POST'
            ' data')


class CreateImageView(AjaxResponseMixin, UserMediaImageViewMixin, CreateView):
    action = 'create'
    form_class = UserMediaImageForm
    ajax_template_prefix = 'partials/ajax_'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """Adds useful objects to the class and performs security checks."""
        self._add_next_and_user(request)
        self.content_object = None
        self.content_type = None
        self.object_id = kwargs.get('object_id', None)

        if kwargs.get('content_type'):
            # Check if the user forged the URL and posted a non existant
            # content type
            try:
                self.content_type = ContentType.objects.get(
                    model=kwargs.get('content_type'))
            except ContentType.DoesNotExist:
                raise Http404

        if self.content_type:
            # Check if the user forged the URL and tries to append the image to
            # an object that does not exist
            try:
                self.content_object = \
                    self.content_type.get_object_for_this_type(
                        pk=self.object_id)
            except ObjectDoesNotExist:
                raise Http404

        if self.content_object and hasattr(self.content_object, 'user'):
            # Check if the user forged the URL and tries to append the image to
            # an object that does not belong to him
            if not self.content_object.user == self.user:
                raise Http404

        return super(CreateImageView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(CreateImageView, self).get_context_data(**kwargs)
        ctx.update({
            'content_type': self.content_type,
            'object_id': self.object_id,
        })
        return ctx

    def get_form_kwargs(self):
        kwargs = super(CreateImageView, self).get_form_kwargs()
        kwargs.update({
            'user': self.user,
            'content_type': self.content_type,
            'object_id': self.object_id,
        })
        return kwargs


class DeleteImageView(AjaxResponseMixin, UserMediaImageViewMixin, DeleteView):
    """Deletes an `UserMediaImage` object."""
    action = 'delete'
    model = UserMediaImage
    ajax_template_prefix = 'partials/ajax_'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """Adds useful objects to the class."""
        self._add_next_and_user(request)
        return super(DeleteImageView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(DeleteImageView, self).get_context_data(**kwargs)
        ctx.update({
            'image_pk': self.object.pk,
        })
        return ctx

    def get_queryset(self):
        """
        Making sure that a user can only delete his own images.

        Even when he forges the request URL.

        """
        queryset = super(DeleteImageView, self).get_queryset()
        queryset = queryset.filter(user=self.user)
        return queryset


class UpdateImageView(AjaxResponseMixin, UserMediaImageViewMixin, UpdateView):
    """Updates an existing `UserMediaImage` object."""
    action = 'update'
    model = UserMediaImage
    form_class = UserMediaImageForm
    ajax_template_prefix = 'partials/ajax_'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        """Adds useful objects to the class."""
        self._add_next_and_user(request)
        return super(UpdateImageView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        ctx = super(UpdateImageView, self).get_context_data(**kwargs)
        ctx.update({
            'content_type': self.object.content_type,
            'object_id': self.object.object_id,
            'image_pk': self.object.pk,
        })
        return ctx

    def get_form_kwargs(self):
        kwargs = super(UpdateImageView, self).get_form_kwargs()
        kwargs.update({
            'user': self.user,
            'content_type': self.object.content_type,
            'object_id': self.object.object_id,
        })
        return kwargs

    def get_queryset(self):
        """
        Making sure that a user can only edit his own images.

        Even when he forges the request URL.

        """
        queryset = super(UpdateImageView, self).get_queryset()
        queryset = queryset.filter(user=self.user)
        return queryset


class AJAXMultipleImageUploadView(CreateView):
    """Ajax view to handle the multiple image upload."""
    model = UserMediaImage
    form_class = UserMediaImageForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.obj_id = kwargs.get('obj_id', None)
        self.user = request.user

        if not request.is_ajax():
            # Since we use a jquery modal and a jquery upload we should only
            # allow ajax calls
            raise Http404

        # Check if the user posted a non existant content type
        try:
            self.c_type = ContentType.objects.get(model=kwargs.get('c_type'))
        except ContentType.DoesNotExist:
            raise Http404

        # Check if the content object exists
        try:
            self.content_object = self.c_type.get_object_for_this_type(
                pk=self.obj_id)
        except ObjectDoesNotExist:
            raise Http404

        # Check for permissions
        # Add a single user to the content object or prepare a user_can_edit
        # function.
        if (not hasattr(self.content_object, 'user') or not
                self.content_object.user == self.user):
            if (not hasattr(self.content_object, 'user_can_edit') or not
                    self.content_object.user_can_edit(self.user)):
                raise Http404
        return super(AJAXMultipleImageUploadView, self).dispatch(
            request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(AJAXMultipleImageUploadView, self).get_form_kwargs()
        # Prepare context for UserMediaImage form
        kwargs.update({
            'user': self.user,
            'content_type': self.c_type,
            'object_id': self.obj_id,
        })
        return kwargs

    def form_valid(self, form):
        # Check if maximum amount of pictures has been reached
        try:
            max_pictures = int(self.request.POST.get('maximum'))
        except (TypeError, ValueError):
            max_pictures = getattr(settings, 'USER_MEDIA_UPLOAD_MAXIMUM', 3)
        stored_images = self.user.usermediaimage_set.filter(
            object_id=self.obj_id, content_type=self.c_type)
        if stored_images.count() >= max_pictures:
            return HttpResponse(_('Maximum amount limit exceeded.'))

        # Save the UserMediaImage
        self.object = form.save()
        f = self.request.FILES.get('image')

        # Generate and get the thumbnail of the uploaded image
        thumbnailer = get_thumbnailer(self.object.image.name)
        thumb = thumbnailer.get_thumbnail({
            'crop': True, 'upscale': True,
            'size': self.object.small_size(as_string=False),
        })

        # Prepare context for the list item html
        context = {
            'image': self.object,
            'mode': 'multiple',
        }
        # Prepare the json response
        data = {'files': [{
            'name': f.name,
            'url': self.object.image.url,
            'thumbnail_url': thumb.url,
            'list_item_html': render_to_string(
                'user_media/partials/image.html', context=context,
                request=self.request),
        }]}
        response = HttpResponse(dumps(data), content_type='application/json')
        response['Content-Disposition'] = 'inline; filename=files.json'
        return response


class AJAXSingleImageUploadView(FormView):
    """Ajax view to handle the single image upload."""
    form_class = UserMediaImageSingleUploadForm
    template_name = 'user_media/partials/image.html'

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.is_ajax() or not request.method == 'POST':
            raise Http404
        self.user = request.user
        # Check if the user posted a non existant content type
        try:
            self.c_type = ContentType.objects.get(model=kwargs.get('c_type'))
        except ContentType.DoesNotExist:
            raise Http404

        # Check if the content object exists
        try:
            self.content_object = self.c_type.get_object_for_this_type(
                pk=kwargs.get('obj_id'))
        except ObjectDoesNotExist:
            raise Http404

        # Check if content_object has the requested image field
        if hasattr(self.content_object, kwargs.get('field')):
            self.field_name = kwargs.get('field')
        else:
            raise Http404

        # Check for permissions
        if (not hasattr(self.content_object, 'user') or not
                self.content_object.user == self.user):
            if (not hasattr(self.content_object, 'user_can_edit') or not
                    self.content_object.user_can_edit(self.user)):
                raise Http404
        return super(AJAXSingleImageUploadView, self).dispatch(
            request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(AJAXSingleImageUploadView, self).get_form_kwargs()
        kwargs.update({
            'instance': self.content_object,
            'image_field': self.field_name,
        })
        return kwargs

    def form_valid(self, form):
        # Save the image
        self.content_object = form.save()
        f = self.request.FILES.get(self.field_name)
        image = getattr(self.content_object, self.field_name)
        size = getattr(settings, 'USER_MEDIA_THUMB_SIZE_LARGE', (150, 150))

        # Generate and get the thumbnail of the uploaded image
        thumbnailer = get_thumbnailer(image.name)
        thumb = thumbnailer.get_thumbnail({
            'crop': True, 'upscale': True,
            'size': size,
        })

        # Prepare context for the list item html
        context = {
            'image': image,
            'mode': 'single',
            'size': (self.request.POST.get('size') or
                     u'{}x{}'.format(size[0], size[1])),
        }
        # Prepare the json response
        data = {'files': [{
            'name': f.name,
            'url': image.url,
            'thumbnail_url': thumb.url,
            'list_item_html': render_to_string(
                self.template_name, context=context, request=self.request),
        }]}
        response = HttpResponse(dumps(data), content_type='application/json')
        response['Content-Disposition'] = 'inline; filename=files.json'
        return response


class AJAXImageCropView(UserMediaImageViewMixin, UpdateView):
    """Ajax view to update an image's crop attributes."""
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.is_ajax() or not request.method == 'POST':
            raise Http404
        self.user = request.user
        self.kwargs = kwargs
        self.object = self.get_object()
        if not self.object.user == self.user:
            raise Http404
        for field in ['x', 'x2', 'y', 'y2', 'w', 'h']:
            # Save the Jcrop values to the image
            setattr(self.object, 'thumb_' + field, request.POST.get(field))
        self.object.save()

        box = (
            int(self.object.thumb_x),
            int(self.object.thumb_y),
            int(self.object.thumb_x2),
            int(self.object.thumb_y2),
        )
        thumbnailer = get_thumbnailer(self.object.image.name)
        thumb = thumbnailer.get_thumbnail({
            'box': box,
            'size': self.object.small_size(as_string=False),
        })
        return HttpResponse(thumb.url)



"""Tests for the forms of the ``django-user-media`` app."""
from django.test import TestCase

from mixer.backend.django import mixer, get_image

from ..forms import UserMediaImageForm, UserMediaImageSingleUploadForm
from .test_app.forms import DummyModelForm


class UserMediaImageFormMixinTestCase(TestCase):
    """Tests for the ``UserMediaImageFormMixin`` form mixin."""
    def test_mixin(self):
        form = DummyModelForm(
            instance=mixer.blend('test_app.DummyModel'),
            data={'user': mixer.blend('auth.User').pk},
            files={'images': get_image()})
        self.assertTrue(form.is_valid(), msg=(
            'Should be valid, but returned: {}'.format(
                form.errors.items())))
        form.save()


class UserMediaImageFormTestCase(TestCase):
    """Tests for the ``UserMediaImageForm`` model form."""
    longMessage = True

    def test_form(self):
        form = UserMediaImageForm(
            mixer.blend('auth.User'), None, None,
            files={'image': get_image()})
        self.assertTrue(form.is_valid(), msg=(
            'Should be valid but returned: %s' % form.errors.items()))
        result = form.save()
        self.assertIn('.jpg', result.image.url)


class UserMediaImageSingleUploadFormTestCase(TestCase):
    """Tests for the ``UserMediaImageSingleUploadForm`` model."""
    longMessage = True

    def test_form(self):
        form = UserMediaImageSingleUploadForm(
            instance=mixer.blend('test_app.DummyModel'), image_field='image',
            data={})
        self.assertFalse(form.is_valid())



"""URLs for the ``django-user-media`` app."""
from django.conf.urls import url

from .views import (
    AJAXImageCropView,
    AJAXMultipleImageUploadView,
    AJAXSingleImageUploadView,
    CreateImageView,
    DeleteImageView,
    UpdateImageView,
)


urlpatterns = [
    url(r'^image/create/$',
        CreateImageView.as_view(),
        name='user_media_image_create_no_ctype'),
    url(r'^image/(?P<content_type>[-\w]+)/(?P<object_id>\d+)/create/$',
        CreateImageView.as_view(),
        name='user_media_image_create'),
    url(r'^image/(?P<pk>\d+)/$',
        UpdateImageView.as_view(),
        name='user_media_image_edit'),
    url(r'^image/(?P<pk>\d+)/crop/$',
        AJAXImageCropView.as_view(),
        name='user_media_image_crop'),
    url(r'^image/(?P<pk>\d+)/delete/$',
        DeleteImageView.as_view(),
        name='user_media_image_delete'),
    url(r'^upload-single/(?P<c_type>[-\w]+)/(?P<obj_id>\d+)/(?P<field>\w+)/$',
        AJAXSingleImageUploadView.as_view(),
        name='user_media_ajax_single_upload'),
    url(r'^upload-multiple/(?P<c_type>[-\w]+)/(?P<obj_id>\d+)/$',
        AJAXMultipleImageUploadView.as_view(),
        name='user_media_ajax_multiple_upload'),
]



"""Forms for the ``django-user-media`` app."""
from django import forms
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from user_media.models import UserMediaImage


class UserMediaImageFormMixin(object):
    """
    Adds an `forms.ImageField` with name `user_media_image` to the form.

    Overrides `save()` and makes sure that the uploaded image get's tied
    to the content object instance.

    This is useful if you have a model form for your content object and you
    want to support uploading the user media image right from that form.

    Please make sure that your content object has a property called `user` that
    returns the user to which the content object belongs to.

    Currently it only supports one image per content object. On each subsequent
    upload, all other images of that content object will be deleted before the
    new image will be saved.

    The field name of the ImageField can be overridden by setting
    ``image_field_name`` on the form, that includes this mixin.

    """

    image_label = _('Image')
    require_user_media_image = False
    image_field_name = 'user_media_image'
    image_widget = forms.ClearableFileInput()

    def __init__(self, *args, **kwargs):
        super(UserMediaImageFormMixin, self).__init__(*args, **kwargs)
        try:
            initial_image = getattr(
                self.instance, self.image_field_name).order_by(
                    '-id')[0].image
        except IndexError:
            initial_image = None
        self.fields[self.image_field_name] = forms.ImageField(
            required=self.require_user_media_image,
            label=self.image_label,
            initial=initial_image,
            widget=self.image_widget,
        )

    def _delete_images(self, instance):
        """Deletes all user media images of the given instance."""
        UserMediaImage.objects.filter(
            content_type=ContentType.objects.get_for_model(instance),
            object_id=instance.pk,
            user=instance.user,
        ).delete()

    def save(self, *args, **kwargs):
        instance = super(UserMediaImageFormMixin, self).save(*args, **kwargs)
        umedia_image = self.cleaned_data[self.image_field_name]
        if umedia_image:
            self._delete_images(instance)
            image = UserMediaImage()
            image.user = instance.user
            image.content_type = ContentType.objects.get_for_model(
                instance)
            image.object_id = instance.pk
            image.image = umedia_image
            image.save()
        return instance


class UserMediaImageForm(forms.ModelForm):
    """Form that allows to create or update an `UserMediaImage` object."""
    class Meta:
        model = UserMediaImage
        fields = ('image', )

    def __init__(self, user, content_type=None, object_id=None,
                 *args, **kwargs):
        self.user = user
        self.content_type = content_type
        self.object_id = object_id
        if content_type is not None and object_id is not None:
            self.content_object = content_type.get_object_for_this_type(
                pk=self.object_id)
        super(UserMediaImageForm, self).__init__(*args, **kwargs)

    def clean_image(self):
        """
        It seems like in Django 1.5 something has changed.

        When Django tries to validate the form, it checks if the generated
        filename fit into the max_length. But at this point, self.instance.user
        is not yet set so our filename generation function cannot create
        the new file path because it needs the user id. Setting
        self.instance.user at this point seems to work as a workaround.

        """
        self.instance.user = self.user
        data = self.cleaned_data.get('image')
        return data

    def save(self, *args, **kwargs):
        self.instance.user = self.user
        if self.content_type is not None and self.object_id is not None:
            self.instance.content_type = self.content_type
            self.instance.object_id = self.object_id
        return super(UserMediaImageForm, self).save(*args, **kwargs)


class UserMediaImageSingleUploadForm(forms.ModelForm):
    """Form to save a single image upload."""
    def __init__(self, image_field, *args, **kwargs):
        self._meta.model = type(kwargs['instance'])
        super(UserMediaImageSingleUploadForm, self).__init__(*args, **kwargs)
        self.fields[image_field] = forms.ImageField()



"""Settings that need to be set in order to run the tests."""
import os

DJANGO_PROJECT_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..'))

DEBUG = True
SITE_ID = 1

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'user_media.tests.urls'

MEDIA_URL = '/media/'
STATIC_URL = '/static/'
MEDIA_ROOT = os.path.join(DJANGO_PROJECT_ROOT, '../media')
STATIC_ROOT = os.path.join(DJANGO_PROJECT_ROOT, '../static')
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(DJANGO_PROJECT_ROOT, 'static'),
)

TEMPLATES = [{
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'APP_DIRS': True,
    'DIRS': [os.path.join(os.path.dirname(__file__), '../templates')],
    'OPTIONS': {
        'context_processors': (
            'django.contrib.auth.context_processors.auth',
            'django.template.context_processors.i18n',
            'django.template.context_processors.request',
            'django.template.context_processors.media',
            'django.template.context_processors.static',
        )
    }
}]

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'easy_thumbnails',
    'django_libs',
    'generic_positions',
]

INTERNAL_APPS = [
    'user_media',
    'user_media.tests.test_app',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

SECRET_KEY = 'xxx'

THUMBNAIL_PROCESSORS = (
    'user_media.processors.crop_box',
    'easy_thumbnails.processors.colorspace',
    'easy_thumbnails.processors.autocrop',
    'easy_thumbnails.processors.scale_and_crop',
    'easy_thumbnails.processors.filters',
)



"""Admin classes for the ``django-user-media`` app."""
from django.contrib import admin

from user_media.models import UserMediaImage


class UserMediaImageAdmin(admin.ModelAdmin):
    list_display = ('user_email', 'content_type', 'object_id', 'image', )
    list_filter = ('content_type', )
    search_fields = ('user__email', )

    def user_email(self, obj):
        return obj.user.email
    user_email.short_description = 'User'


admin.site.register(UserMediaImage, UserMediaImageAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-05 12:55
from __future__ import unicode_literals

from django.db import migrations, models
import user_media.models


class Migration(migrations.Migration):

    dependencies = [
        ('user_media', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usermediaimage',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=user_media.models.get_image_file_path, verbose_name='Image'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings

class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserMediaImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField(null=True, blank=True)),
                ('image', models.ImageField(upload_to='images', null=True, verbose_name='Image', blank=True)),
                ('thumb_x', models.PositiveIntegerField(null=True, verbose_name='Thumbnail x', blank=True)),
                ('thumb_x2', models.PositiveIntegerField(null=True, verbose_name='Thumbnail x2', blank=True)),
                ('thumb_y', models.PositiveIntegerField(null=True, verbose_name='Thumbnail y', blank=True)),
                ('thumb_y2', models.PositiveIntegerField(null=True, verbose_name='Thumbnail y2', blank=True)),
                ('thumb_w', models.PositiveIntegerField(null=True, verbose_name='Thumbnail width', blank=True)),
                ('thumb_h', models.PositiveIntegerField(null=True, verbose_name='Thumbnail height', blank=True)),
                ('content_type', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True)),
                ('user', models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



"""Tests for the models of the ``django-user-media`` app."""
from django.test import TestCase

from mixer.backend.django import mixer


class UserMediaImageTestCase(TestCase):
    """Tests for the ``UserMediaImage`` model."""
    def setUp(self):
        self.image = mixer.blend('user_media.UserMediaImage')

    def test_box_coordinates(self):
        self.assertFalse(self.image.box_coordinates, msg=(
            'Should return False, if there are no coordinates saved.'))
        self.image.thumb_x = 10
        self.image.thumb_y = 10
        self.image.thumb_x2 = 20
        self.assertFalse(self.image.box_coordinates, msg=(
            'Should return False, if there are not enough coordinates saved.'))
        self.image.thumb_y2 = 20
        self.assertEqual(self.image.box_coordinates, (10, 10, 20, 20), msg=(
            'Should return the coordinates in a tuple.'))

    def test_small_size(self):
        self.assertEqual(self.image.small_size(), '95x95', msg=(
            'Should return a size tuple.'))
        self.assertEqual(self.image.small_size(as_string=False), (95, 95),
                         msg='Should return a size tuple.')

    def test_large_size(self):
        self.assertEqual(self.image.large_size(), '150x150', msg=(
            'Should return a size tuple.'))
        self.assertEqual(self.image.large_size(as_string=False), (150, 150),
                         msg='Should return a size tuple.')



"""Models for the ``django-user-media`` app."""
import glob
import os
import uuid

from django.conf import settings
from django.contrib.contenttypes import fields
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.signals import post_delete
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _


def get_image_file_path(instance, filename):
    """Returns a unique filename for images."""
    ext = filename.split('.')[-1]
    filename = '%s.%s' % (uuid.uuid4(), ext)
    return os.path.join(
        'user_media', str(instance.user.pk), 'images', filename)


class UserMediaImage(models.Model):
    """
    An image that can be uploaded by a user.

    If the image belongs to a certain object that is owned by the user, it
    can be tied to that object using the generic foreign key. That object
    must have a foreign key to ``auth.User`` (or another user model) and that
    field must be called ``user``.

    :user: The user this image belongs to.
    :content_type: If this image belongs to a certain object (i.e. a Vehicle),
      this should be the object's ContentType.
    :object_id: If this image belongs to a certain object (i.e. a Vehicle),
      this should be the object's ID.
    :image: The uploaded image.
    :position: The position of the image in case of multiple ones.
    :thumb_x: Thumbnail starting point on the x-axis.
    :thumb_x2: Thumbnail ending point on the x-axis.
    :thumb_y: Thumbnail starting point on the y-axis.
    :thumb_y2: Thumbnail ending point on the y-axis.
    :thumb_w: Thumbnail width.
    :thumb_h: Thumbnail height.

    """
    user = models.ForeignKey(
        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        verbose_name=_('User'),
        on_delete=models.CASCADE,
    )

    content_type = models.ForeignKey(
        ContentType,
        null=True, blank=True,
        on_delete=models.CASCADE,
    )

    object_id = models.PositiveIntegerField(
        null=True, blank=True
    )

    content_object = fields.GenericForeignKey('content_type', 'object_id')

    image = models.ImageField(
        upload_to=get_image_file_path,
        null=True, blank=True,
        verbose_name=_('Image'),
    )

    generic_position = fields.GenericRelation(
        'generic_positions.ObjectPosition'
    )

    thumb_x = models.PositiveIntegerField(
        verbose_name=_('Thumbnail x'),
        null=True, blank=True,
    )

    thumb_x2 = models.PositiveIntegerField(
        verbose_name=_('Thumbnail x2'),
        null=True, blank=True,
    )

    thumb_y = models.PositiveIntegerField(
        verbose_name=_('Thumbnail y'),
        null=True, blank=True,
    )

    thumb_y2 = models.PositiveIntegerField(
        verbose_name=_('Thumbnail y2'),
        null=True, blank=True,
    )

    thumb_w = models.PositiveIntegerField(
        verbose_name=_('Thumbnail width'),
        null=True, blank=True,
    )

    thumb_h = models.PositiveIntegerField(
        verbose_name=_('Thumbnail height'),
        null=True, blank=True,
    )

    @property
    def box_coordinates(self):
        """Returns a thumbnail's coordinates."""
        if (
            self.thumb_x is not None and
            self.thumb_y is not None and
            self.thumb_x2 is not None and
            self.thumb_y2 is not None
        ):
            return (
                int(self.thumb_x),
                int(self.thumb_y),
                int(self.thumb_x2),
                int(self.thumb_y2),
            )
        return False

    def large_size(self, as_string=True):
        """Returns a thumbnail's large size."""
        size = getattr(settings, 'USER_MEDIA_THUMB_SIZE_LARGE', (150, 150))
        if as_string:
            return u'{}x{}'.format(size[0], size[1])
        return size

    def small_size(self, as_string=True):
        """Returns a thumbnail's small size."""
        size = getattr(settings, 'USER_MEDIA_THUMB_SIZE_SMALL', (95, 95))
        if as_string:
            return u'{}x{}'.format(size[0], size[1])
        return size


@receiver(post_delete, sender=UserMediaImage)
def image_post_delete_handler(sender, instance, **kwargs):
    """
    Makes sure that a an image is also deleted from the media directory.

    This should prevent a load of "dead" image files on disc.

    """
    for f in glob.glob('{}/{}*'.format(instance.image.storage.location,
                                       instance.image.name)):
        if not os.path.isdir(f):
            instance.image.storage.delete(f)



"""Tests for the views of the ``django-user-media`` app."""
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin
from mixer.backend.django import mixer, get_image

from .. import views


class CreateImageViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """
    Tests for the ``CreateImageView`` generic view class.

    Tests the case when the view is called without content type and object
    id.

    """
    view_class = views.CreateImageView

    def setUp(self):
        self.dummy = mixer.blend('test_app.DummyModel')
        self.user = self.dummy.user
        self.other_dummy = mixer.blend('test_app.DummyModel')
        self.gallery = mixer.blend('test_app.DummyGallery')

    def get_view_name(self):
        return 'user_media_image_create'

    def get_view_kwargs(self):
        ctype = ContentType.objects.get_for_model(self.dummy)
        return {
            'content_type': ctype.model,
            'object_id': self.dummy.pk,
        }

    def test_view(self):
        self.is_callable(self.user)

        data = {'image': get_image(), }
        self.is_postable(user=self.user, data=data,
                         to=self.dummy.get_absolute_url())

        data = {'image': get_image(), 'next': '/?foo=bar'}
        self.is_postable(user=self.user, data=data, to='/?foo=bar')

        data = {'image': get_image(), }
        self.is_postable(user=self.user, data=data, to='/?foo=bar')

        self.is_not_callable(post=True, user=self.user, kwargs={
            'content_type': 'dummymodel', 'object_id': 999})

        self.is_not_callable(post=True, user=self.user, kwargs={
            'content_type': 'foobarmodel', 'object_id': 1})

        self.is_not_callable(post=True, user=self.user, kwargs={
            'content_type': 'dummymodel',
            'object_id': self.other_dummy.pk
        })

        self.is_not_callable(user=self.user, kwargs={
            'content_type': ContentType.objects.get_for_model(
                self.gallery).model,
            'object_id': self.gallery.pk,
        })


class CreateImageViewNoCtypeTestCase(ViewRequestFactoryTestMixin, TestCase):
    """
    Tests for the ``CreateImageView`` generic view class.

    Tests the case when the view is called without content type and object
    id.

    """
    view_class = views.CreateImageView

    def setUp(self):
        self.user = mixer.blend('auth.User')

    def get_view_name(self):
        return 'user_media_image_create_no_ctype'

    def test_view(self):
        self.is_callable(self.user)

        data = {'image': get_image(), 'next': '/?foo=bar'}
        self.is_postable(user=self.user, to='/?foo=bar', data=data)

        data = {'image': get_image(), }
        try:
            self.is_postable(user=self.user, data=data)
        except Exception as err:
            self.assertTrue('No content object' in '{}'.format(err), msg=(
                'If no content object and no ``next`` parameter given,'
                ' view should raise an exception'))


class EditAndDeleteTestCaseMixin(object):
    """Tests that are the same for both views."""
    def setUp(self):
        self.dummy = mixer.blend('test_app.DummyModel')
        self.user = self.dummy.user
        self.image = mixer.blend('user_media.UserMediaImage', user=self.user)
        self.image.content_object = self.dummy
        self.image.image = get_image()
        self.image.save()
        self.image_no_content_object = mixer.blend(
            'user_media.UserMediaImage', user=self.user)
        self.other_image = mixer.blend('user_media.UserMediaImage')

    def test_view_with_content_object(self):
        self.is_callable(self.user)
        self.is_postable(
            user=self.user, to=self.dummy.get_absolute_url(), msg=(
                "If the image had a content object, view should redirect to "
                "that object's absolute url"))

        self.image = mixer.blend('user_media.UserMediaImage', user=self.user)
        self.is_postable(
            user=self.user, data={'next': '/?foo=bar'}, to='/?foo=bar', msg=(
                "If the image had a content object and ``next`` in the POST"
                " data, view should redirect to the URL given in ``next`` and"
                " ignore the content object's absolute URL"))

        self.is_not_callable(
            user=self.user, kwargs={'pk': self.other_image.pk}, post=True,
            msg=("If the image had a content object and ``next`` in the POST"
                 " data, view should redirect to the URL given in ``next`` and"
                 " ignore the content object's absolute URL"))

        self.is_not_callable(
            user=self.user, kwargs={'pk': 999}, post=True,
            msg=('Should return 404 if the user tries to manipulate a non'
                 ' existing object'))

    def test_view_without_content_object(self):
        self.is_postable(
            user=self.user, to='/?foo=bar', data={'next': '/?foo=bar'},
            kwargs={'pk': self.image_no_content_object.pk}, msg=(
                'If the image had no content object, view should redirect to'
                ' the POST data ``next`` that must be given'))

        self.image_no_content_object = mixer.blend(
            'user_media.UserMediaImage', user=self.user)
        try:
            self.is_postable(
                user=self.user, to='/?foo=bar', data={'next': '/?foo=bar'},
                kwargs={'pk': self.image_no_content_object.pk})
        except Exception as err:
            self.assertTrue('No content object' in '{}'.format(err), msg=(
                'If no content object and no ``next`` parameter given,'
                ' view should raise an exception'))


class DeleteImageViewTestCase(ViewRequestFactoryTestMixin,
                              EditAndDeleteTestCaseMixin, TestCase):
    """Tests for the ``DeleteImageView`` generic view class."""
    view_class = views.DeleteImageView

    def get_view_name(self):
        return 'user_media_image_delete'

    def get_view_kwargs(self):
        return {'pk': self.image.pk}


class UpdateImageViewTestCase(ViewRequestFactoryTestMixin,
                              EditAndDeleteTestCaseMixin, TestCase):
    """Tests for the ``UpdateImageView`` view class."""
    view_class = views.UpdateImageView

    def get_view_name(self):
        return 'user_media_image_edit'

    def get_view_kwargs(self):
        return {'pk': self.image.pk}


class AJAXMultipleImageUploadViewTestCase(ViewRequestFactoryTestMixin,
                                          TestCase):
    """Tests for the ``AJAXMultipleImageUploadView`` generic view class."""
    view_class = views.AJAXMultipleImageUploadView

    def setUp(self):
        self.profile = mixer.blend('test_app.DummyModel')
        self.other_profile = mixer.blend('test_app.DummyModel')
        self.invalid_content_object = mixer.blend('auth.User')
        self.c_type = ContentType.objects.get_for_model(self.profile).model

    def get_view_name(self):
        return 'user_media_ajax_multiple_upload'

    def get_view_kwargs(self):
        return {'c_type': self.c_type, 'obj_id': self.profile.id}

    def upload_to_gallery(self):
        kwargs = {'c_type': self.c_type, 'obj_id': self.profile.id}
        self.is_postable(
            data={'image': get_image()}, ajax=True, user=self.profile.user,
            kwargs=kwargs, msg=('Upload should be valid.'))

    def test_view(self):
        self.is_not_callable(user=self.profile.user,
                             msg=('Should only be callable via AJAX.'))

        self.is_not_callable(
            user=self.profile.user, ajax=True,
            kwargs={'c_type': 'foo', 'obj_id': self.profile.id},
            msg=('Should only be callable, if content type exists.'))

        self.is_not_callable(
            user=self.profile.user, ajax=True,
            kwargs={'c_type': self.c_type, 'obj_id': 999},
            msg=('Should only be callable, if content object exists.'))

        self.is_not_callable(
            user=self.profile.user, ajax=True,
            kwargs={'c_type': self.c_type, 'obj_id': self.other_profile.pk},
            msg=('Should only be callable, if the current user owns the'
                 ' chosen profile.'))

        self.is_not_callable(
            user=self.profile.user, ajax=True,
            kwargs={
                'c_type': ContentType.objects.get_for_model(
                    self.invalid_content_object),
                'obj_id': self.invalid_content_object.pk
            },
            msg=("Should only be callable, if the content object is one of"
                 " the user's items."))

        self.upload_to_gallery()
        self.upload_to_gallery()
        self.upload_to_gallery()
        self.upload_to_gallery()
        self.upload_to_gallery()

        resp = self.is_postable(
            data={'image': get_image()}, ajax=True, user=self.profile.user,
            msg=('Upload should be valid.'))
        self.assertEqual(resp.content, b'Maximum amount limit exceeded.', msg=(
            'Should return an error message.'))


class AJAXSingleImageUploadViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``AJAXSingleImageUploadView`` generic view class."""
    view_class = views.AJAXSingleImageUploadView

    def setUp(self):
        self.gallery = mixer.blend('test_app.DummyGallery')
        self.other_gallery = mixer.blend('test_app.DummyGallery')
        self.invalid_content_object = mixer.blend('auth.User')
        self.c_type = ContentType.objects.get_for_model(self.gallery).model

    def get_view_name(self):
        return 'user_media_ajax_single_upload'

    def get_view_kwargs(self):
        return {
            'c_type': self.c_type,
            'obj_id': self.gallery.id,
            'field': 'logo',
        }

    def test_view(self):
        self.is_not_callable(post=True, user=self.gallery.user_connection,
                             msg=('Should only be callable via AJAX.'))

        self.is_postable(
            data={'logo': get_image()}, ajax=True,
            user=self.gallery.user_connection, msg=('Upload should be valid.'))

        new_kwargs = {
            'c_type': 'foo',
            'obj_id': self.gallery.id,
            'field': 'logo',
        }
        self.is_not_callable(
            ajax=True, post=True, kwargs=new_kwargs,
            user=self.gallery.user_connection,
            msg=('Should only be callable, if content type exists.'))

        new_kwargs = {
            'c_type': self.c_type,
            'obj_id': self.gallery.id,
            'field': 'foobar',
        }
        self.is_not_callable(
            ajax=True, post=True, kwargs=new_kwargs,
            user=self.gallery.user_connection,
            msg=('Should only be callable, if field exists.'))

        new_kwargs = {
            'c_type': self.c_type,
            'obj_id': '999',
            'field': 'logo',
        }
        self.is_not_callable(
            ajax=True, post=True, kwargs=new_kwargs,
            user=self.gallery.user_connection,
            msg=('Should only be callable, if content object exists.'))

        new_kwargs = {
            'c_type': self.c_type,
            'obj_id': self.other_gallery.pk,
            'field': 'logo',
        }
        self.is_not_callable(
            ajax=True, post=True, kwargs=new_kwargs,
            user=self.gallery.user_connection,
            msg=('Should only be callable, if the current user owns the'
                 ' chosen vendor.'))


class AJAXImageCropViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``AJAXImageCropView`` generic view class."""
    view_class = views.AJAXImageCropView

    def setUp(self):
        self.image = mixer.blend('user_media.UserMediaImage')
        self.image2 = mixer.blend('user_media.UserMediaImage')

    def get_view_name(self):
        return 'user_media_image_crop'

    def get_view_kwargs(self):
        return {'pk': self.image.pk}

    def test_view(self):
        self.image.image = get_image()
        self.image.save()
        self.is_not_callable(user=self.image.user)
        data = {'x': 10, 'x2': 15, 'y': 2, 'y2': 10, 'w': 5, 'h': 8}
        self.is_postable(user=self.image.user, data=data, ajax=True)
        self.is_not_callable(user=self.image2.user, ajax=True, post=True)



"""Image processors for easy_thumbnails."""


def crop_box(im, box=False, **kwargs):
    """Uses box coordinates to crop an image without resizing it first."""
    if box:
        im = im.crop(box)
    return im



"""Dummy forms for the tests of the `django-user-media` application."""
from django import forms
from django.utils.translation import ugettext_lazy as _

from user_media.forms import UserMediaImageFormMixin
from user_media.tests.test_app.models import DummyModel


class DummyModelForm(UserMediaImageFormMixin, forms.ModelForm):
    image_label = _('Image')
    require_user_media_image = False
    image_field_name = 'images'

    class Meta:
        model = DummyModel
        fields = ('user', )
